//
//  ViewController.swift
//  ShapesGalore
//
//  See the end of this file for license information.
//

import UIKit
import ARKit

class ViewController: UIViewController {
  
  // MARK: Properties
  
  // Outlets
  @IBOutlet weak var sceneView: ARSCNView!
  
  // When we start the augmented reality (AR) session, we need to specify
  // an AR configuration to use.
  // ARWorldTrackingConfiguration is the full-fidelity configuration
  // that uses the rear camera and does the following:
  //
  // - Tracks the device’s position in 3D space relative to
  //   it starting point (x-, y-, and z-position)
  // - Tracks the device’s orientation (its tilt around the x-,
  //   y-, and z-axes
  // - Detects real-world flat surfaces
  //
  // For this app, we’re concerned only about tracking the device’s
  // position and orientation (a.k.a. the “six degrees of freedom”).
  let configuration = ARWorldTrackingConfiguration()
  
  
  // MARK: Methods
  
  override func viewDidLoad() {
    super.viewDidLoad()
    
    // Seed the random number generator.
    // You only have to do this once per session.
    srand48(Int(Date().timeIntervalSince1970))
    
    // This turns on the two available debugging options:
    //
    // 1. showWorldOrigin, which shows the AR session origin,
    //    which is defined by the phone’s location at the start
    //    of the AR session. The origin is marked by the intersection
    //    of 3 line segments:
    //    - x-axis, denoted by a red line segment
    //    - y-axis, denoted by a green line segment
    //    - z-axis, denoted by a blue line segment
    // 2. showFeaturePoints, which represents points on flat
    //    horizontal surfaces that ARKit uses as reference points
    sceneView.debugOptions = [
      ARSCNDebugOptions.showWorldOrigin,
      ARSCNDebugOptions.showFeaturePoints
    ]
    
    // Turns on a virtual omnidirectional light source located
    // at the same location as the device
    sceneView.autoenablesDefaultLighting = true
    
    // Starts the AR session
    sceneView.session.run(configuration)
  }

  override func didReceiveMemoryWarning() {
    super.didReceiveMemoryWarning()
  }
  
  @IBAction func addButtonPressed(_ sender: Any) {
    addShape()
  }
  
  func addShape() {
    let node = SCNNode()
    node.geometry = SCNCone(topRadius: 0, bottomRadius: 0.06, height: 0.1)
    node.geometry?.firstMaterial?.diffuse.contents = UIColor.orange
    node.geometry?.firstMaterial?.specular.contents = UIColor.white
    
    // Give the shape a random location
    let x = drand48() * 1 - 0.5
    let y = drand48() * 1 - 0.5
    let z = drand48() * 1 - 0.5
    print("x:\(x) y:\(y) z:\(z)")
    node.position = SCNVector3(x, y, z)
    
    // Give the shape a random orientation
    let eulerX = drand48() * 2 * .pi
    let eulerY = drand48() * 2 * .pi
    let eulerZ = drand48() * 2 * .pi
    print("eulerX:\(eulerX) eulerY:\(eulerY) eulerZ:\(eulerZ)")
    node.eulerAngles = SCNVector3(eulerX, eulerY, eulerZ)
    
    sceneView.scene.rootNode.addChildNode(node)
  }
  
}


// This code is released under the MIT license.
// Simply put, you're free to use this in your own projects, both
// personal and commercial, as long as you include the copyright notice below.
// It would be nice if you mentioned my name somewhere in your documentation
// or credits.
//
// MIT LICENSE
// -----------
// (As defined in https://opensource.org/licenses/MIT)
//
// Copyright © 2017 Joey deVilla. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom
// the Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
// OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
